##############################################################################
#
# Copyright (c) 2010, 2011, 2012 Vifib SARL and Contributors.
# All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly advised to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
"""Exposed exceptions"""


class PathDoesNotExistError(Exception):
  pass


class WrongPermissionError(Exception):
  pass


class BuildoutFailedError(Exception):
  pass


class DiskSpaceError(Exception):
  pass




##############################################################################
# coding: utf-8
# Copyright (c) 2018 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
from __future__ import unicode_literals
import logging
import os
import subprocess
import sys
import tempfile
import textwrap
import unittest

import mock
import slapos.grid.utils


class SlapPopenTestCase(unittest.TestCase):
  def setUp(self):
    self.script = tempfile.NamedTemporaryFile(delete=False)
    # make executable
    os.chmod(self.script.name, 0o777)

  def tearDown(self):
    os.unlink(self.script.name)

  def test_exec(self):
    """Test command execution with SlapPopen.
    """
    self.script.write(b'#!/bin/sh\necho "hello"\nexit 123')
    self.script.close()

    logger = mock.MagicMock()
    program = slapos.grid.utils.SlapPopen(
        self.script.name,
        logger=logger)

    # error code and output are returned
    self.assertEqual(123, program.returncode)
    self.assertEqual('hello\n', program.output)

    # output is also logged "live"
    logger.info.assert_called_once_with('hello')

  def test_exec_multiline(self):
    self.script.write(textwrap.dedent('''\
      #!/bin/sh
      echo -n he
      echo -n l
      sleep 0.1
      echo lo
      echo world
    ''').encode())
    self.script.close()

    logger = mock.MagicMock()
    program = slapos.grid.utils.SlapPopen(
        self.script.name,
        logger=logger)

    self.assertEqual('hello\nworld\n', program.output)
    self.assertEqual(logger.info.call_args_list, [mock.call('hello'), mock.call('world')])

  def test_exec_large_output_multiline(self):
    output_line_list = [str(x) for x in range(100)]
    self.script.write((
      '\n'.join(
        ['#!/bin/sh']
        + ['echo ' + x for x in output_line_list]
      )).encode())
    self.script.close()

    logger = mock.MagicMock()
    program = slapos.grid.utils.SlapPopen(
        self.script.name,
        logger=logger)

    self.assertEqual("\n".join(output_line_list) + '\n', program.output)
    self.assertEqual(
      logger.info.call_args_list,
      [mock.call(x) for x in output_line_list])

  def test_exec_non_ascii(self):
    self.script.write(b'#!/bin/sh\necho h\xc3\xa9h\xc3\xa9')
    self.script.close()

    logger = mock.MagicMock()
    program = slapos.grid.utils.SlapPopen(
        self.script.name,
        logger=logger)

    self.assertEqual('héhé\n', program.output)
    logger.info.assert_called_once_with('héhé')

  def test_exec_large_output(self):
    large_output = 'x' * 10000
    self.script.write(('#!/bin/sh\necho %s' % large_output).encode())
    self.script.close()

    logger = mock.MagicMock()
    program = slapos.grid.utils.SlapPopen(
        self.script.name,
        logger=logger)

    self.assertEqual(large_output + '\n', program.output)
    logger.info.assert_called_once_with(large_output)

  def test_debug(self):
    """Test debug=True, which keeps interactive.
    """
    self.script.write(b'#!/bin/sh\necho "exit code?"\nread rc\nexit $rc')
    self.script.close()

    # when running under pytest we want to disable capture
    with mock.patch.object(sys, 'stdin', sys.__stdin__), \
        mock.patch.object(sys, 'stdout', sys.__stdout__):

      # keep a reference to stdin and stdout to restore them later
      stdin_backup = os.dup(sys.stdin.fileno())
      stdout_backup = os.dup(sys.stdout.fileno())

      # replace stdin with a pipe that will write 123
      child_stdin_r, child_stdin_w = os.pipe()
      os.write(child_stdin_w, b"123")
      os.close(child_stdin_w)
      os.dup2(child_stdin_r, sys.stdin.fileno())

      # and stdout with the pipe to capture output
      child_stdout_r, child_stdout_w = os.pipe()
      os.dup2(child_stdout_w, sys.stdout.fileno())

      try:
        program = slapos.grid.utils.SlapPopen(
            self.script.name,
            debug=True,
            logger=logging.getLogger())
        # program output
        self.assertEqual(b'exit code?\n', os.read(child_stdout_r, 1024))

        self.assertEqual(123, program.returncode)
        self.assertEqual('(output not captured in debug mode)', program.output)
      finally:
        # restore stdin & stderr
        os.dup2(stdin_backup, sys.stdin.fileno())
        os.dup2(stdout_backup, sys.stdout.fileno())
        # close all fds open for the test
        for fd in (child_stdin_r, child_stdout_r, child_stdout_w, stdin_backup, stdout_backup):
          os.close(fd)


class DummySystemExit(Exception):
  """Dummy exception raised instead of SystemExit so that if something goes
  wrong with TestSetRunning we don't exit the test program.
  """
  pass


class TestSetRunning(unittest.TestCase):
  def setUp(self):
    sys_exit_patcher = mock.patch(
        'slapos.grid.utils.sys.exit',
        side_effect=DummySystemExit)
    sys_exit_patcher.start()
    self.addCleanup(sys_exit_patcher.stop)
    self.logger = mock.MagicMock()

  def test_write_pidfile(self):
    with tempfile.NamedTemporaryFile(suffix='.pid', mode='r') as tf:
      slapos.grid.utils.setRunning(self.logger, tf.name)
      self.assertEqual(tf.read(), str(os.getpid()))

  def test_already_running(self):
    process = subprocess.Popen([sys.executable, '-c', 'print("this is fake slapos node"); import time; time.sleep(10)'])
    pid = process.pid
    self.addCleanup(process.wait)
    self.addCleanup(process.terminate)

    with tempfile.NamedTemporaryFile(suffix='.pid', mode='w') as tf:
      tf.write(str(pid))
      tf.flush()
      with self.assertRaises(DummySystemExit):
        slapos.grid.utils.setRunning(self.logger, tf.name)
      self.logger.info.assert_called_with(
          'New slapos process started, but another slapos process '
          'is aleady running with pid %s, exiting.',
          pid)

  def test_stale_pidfile(self):
    # XXX we can not reliably guess a pid that will not be used by a running
    # process. We start a process, record its pid and wait for process to
    # terminate and assume that this pid will not be reused in the meantime.
    process = subprocess.Popen(['sleep', '0.0001'])
    pid = process.pid
    process.wait()

    with tempfile.NamedTemporaryFile(suffix='.pid', mode='w') as tf:
      tf.write(str(pid))
      tf.flush()
      slapos.grid.utils.setRunning(self.logger, tf.name)
      self.logger.info.assert_called_with(
          'Existing pid file %r was stale, overwritten',
          tf.name)
      with open(tf.name) as f:
        self.assertEqual(f.read(), str(os.getpid()))

  def test_stale_pidfile_pid_recycled(self):
    # another unrelated process is running with the pid from the pid file
    process = subprocess.Popen(['sleep', '10'])
    pid = process.pid
    self.addCleanup(process.wait)
    self.addCleanup(process.terminate)

    with tempfile.NamedTemporaryFile(suffix='.pid', mode='w') as tf:
      tf.write(str(pid))
      tf.flush()
      slapos.grid.utils.setRunning(self.logger, tf.name)
      self.logger.info.assert_called_with(
          'Existing pid file %r was stale, overwritten',
          tf.name)
      with open(tf.name) as f:
        self.assertEqual(f.read(), str(os.getpid()))



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010-2014 Vifib SARL and Contributors.
# All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import os
import logging
from datetime import datetime, timedelta
from slapos.collect.snapshot import FolderSizeSnapshot

logger = logging.getLogger(__name__)

def get_user_list(config):
  nb_user = int(config.get("slapformat", "partition_amount"))
  name_prefix = config.get("slapformat", "user_base_name")
  path_prefix = config.get("slapformat", "partition_base_name")
  instance_root = config.get("slapos", "instance_root")
  # By default, enable disk snapshot,
  # and set time_cycle to 24hours after the first disk snapshot run
  pid_folder_tmp = instance_root + "/var/run"
  disk_snapshot_params = {'enable': True, 
                          'time_cycle': 86400,
                          'pid_folder': pid_folder_tmp, 
                          'use_quota': False}

  if config.has_section('collect'):
    collect_section = dict(config.items("collect"))
    disk_snapshot_params = dict(
      enable = collect_section.get("report_disk_usage", "True").lower() in ('true', 'on', '1'),
      pid_folder = collect_section.get("disk_snapshot_process_pid_foder", pid_folder_tmp),
      time_cycle = int(collect_section.get("disk_snapshot_time_cycle", 86400)),
      use_quota = collect_section.get("disk_snapshot_use_quota", "False").lower() in ('true', 'on', '1'),
    )
  user_dict = {name: User(name, path, disk_snapshot_params)
      for name, path in [
          (
            "%s%s" % (name_prefix, nb),
            "%s/%s%s" % (instance_root, path_prefix, nb)
          ) for nb in range(nb_user)
        ]
      }

  #user_dict['root'] = User("root", "/opt/slapgrid")
  return user_dict

class User(object):
  def __init__(self, name, path, disk_snapshot_params={}):
    self.name = str(name)
    self.path = str(path)
    self.disk_snapshot_params = disk_snapshot_params
    self.snapshot_list = []

  def append(self, value):
    self.snapshot_list.append(value)

  def _insertDiskSnapShot(self, database, collected_date, collected_time):
    if self.disk_snapshot_params['enable']:
      time_cycle = self.disk_snapshot_params.get('time_cycle', 0)

      database.connect()
      if time_cycle:
        for date_time in database.select(table="folder", columns="date, time",
                                    order='date DESC, time DESC', limit=1,
                                    where="partition='%s'" % self.name):
          latest_date = datetime.strptime('%s %s' % date_time,
                                          "%Y-%m-%d %H:%M:%S")
          time_spent = (datetime.utcnow() - latest_date).total_seconds()
          if time_spent < time_cycle:
            # wait the time cycle
            logger.info("Time cycle is not over (%s seconds remaining). No computation of " 
                        "disk usage on the partition %s.", time_cycle - time_spent, self.name)
            return
          break
      pid_file = self.disk_snapshot_params.get('pid_folder', None)
      if pid_file is not None:
        pid_file = os.path.join(pid_file, '%s_disk_size.pid' % self.name)

      disk_snapshot = FolderSizeSnapshot(self.path, pid_file)
      disk_snapshot.update_folder_size()
      # Skeep insert empty partition: size <= 1Mb
      if disk_snapshot.disk_usage <= 1024.0 and \
                      not self.disk_snapshot_params.get('testing', False):
        logger.debug("Disk usage of the partition %s: %s. "
                  "Ignoring insertion in the dataset.", self.name, disk_snapshot.disk_usage)
        return
      database.inserFolderSnapshot(self.name,
                disk_usage=disk_snapshot.get("disk_usage"),
                insertion_date=collected_date,
                insertion_time=collected_time)
      database.commit()
      database.close()

  def save(self, database, collected_date, collected_time):
    """ Insert collected data on user collector """
    database.connect()
    for snapshot_item in self.snapshot_list:
      snapshot_item.update_cpu_percent()
      database.insertUserSnapshot(self.name,
            pid=snapshot_item.get("pid"),
            process=snapshot_item.get("process"),
            cpu_percent=snapshot_item.get("cpu_percent"),
            cpu_time=snapshot_item.get("cpu_time"),
            cpu_num_threads=snapshot_item.get("cpu_num_threads"),
            memory_percent=snapshot_item.get("memory_percent"),
            memory_rss=snapshot_item.get("memory_rss"),
            io_rw_counter=snapshot_item.get("io_rw_counter"),
            io_cycles_counter=snapshot_item.get("io_cycles_counter"),
            insertion_date=collected_date, 
            insertion_time=collected_time)
    database.commit()
    database.close()
    # Inser disk snapshot in a new transaction, it can take long
    self._insertDiskSnapShot(database, collected_date, collected_time)
    

class Computer(dict):

  def __init__(self, computer_snapshot):
    self.computer_snapshot = computer_snapshot

  def save(self, database, collected_date, collected_time):
    database.connect()
    self._save_computer_snapshot(database, collected_date, collected_time)
    self._save_system_snapshot(database, collected_date, collected_time)
    self._save_disk_partition_snapshot(database, collected_date, collected_time)
    self._save_temperature_snapshot(database, collected_date, collected_time)
    self._save_heating_snapshot(database, collected_date, collected_time)
    database.commit()
    database.close()

  def _save_computer_snapshot(self, database, collected_date, collected_time):
    partition_list = ";".join(["%s=%s" % (x,y) for x,y in \
                                  self.computer_snapshot.get("partition_list")])
    database.insertComputerSnapshot(
            cpu_num_core=self.computer_snapshot.get("cpu_num_core"), 
            cpu_frequency=self.computer_snapshot.get("cpu_frequency"),
            cpu_type=self.computer_snapshot.get("cpu_type"),
            memory_size=self.computer_snapshot.get("memory_size"),
            memory_type=self.computer_snapshot.get("memory_type"), 
            partition_list=partition_list,
            insertion_date=collected_date, 
            insertion_time=collected_time)

  def _save_system_snapshot(self, database, collected_date, collected_time):
    snapshot = self.computer_snapshot.get("system_snapshot")
    database.insertSystemSnapshot(
      loadavg=snapshot.get("load"),
      cpu_percent=snapshot.get("cpu_percent"), 
      memory_used=snapshot.get("memory_used"), 
      memory_free=snapshot.get("memory_free"),
      net_in_bytes=snapshot.get("net_in_bytes"),
      net_in_errors=snapshot.get("net_in_errors"),
      net_in_dropped=snapshot.get("net_in_dropped"),
      net_out_bytes=snapshot.get("net_out_bytes"),
      net_out_errors= snapshot.get("net_out_errors"),
      net_out_dropped=snapshot.get("net_out_dropped"),
      insertion_date=collected_date, 
      insertion_time=collected_time)

  def _save_disk_partition_snapshot(self, database, collected_date, collected_time):
    for disk_partition in self.computer_snapshot.get("disk_snapshot_list"):
      database.insertDiskPartitionSnapshot(
         partition=disk_partition.partition,
         used=disk_partition.disk_size_used,
         free=disk_partition.disk_size_free,
         mountpoint=';'.join(disk_partition.mountpoint_list),
         insertion_date=collected_date, 
         insertion_time=collected_time)

  def _save_temperature_snapshot(self, database, collected_date, collected_time):
    for temperature_snapshot in self.computer_snapshot.get("temperature_snapshot_list"):
      database.insertTemperatureSnapshot(
         sensor_id=temperature_snapshot.sensor_id,
         temperature=temperature_snapshot.temperature,
         alarm=temperature_snapshot.alarm,
         insertion_date=collected_date, 
         insertion_time=collected_time)

  def _save_heating_snapshot(self, database, collected_date, collected_time):
    heating_snapshot = self.computer_snapshot.get("heating_contribution_snapshot")
    if heating_snapshot is not None and \
         heating_snapshot.initial_temperature is not None:
      database.insertHeatingSnapshot(
         initial_temperature=heating_snapshot.initial_temperature,
         final_temperature=heating_snapshot.final_temperature,
         delta_time=heating_snapshot.delta_time,
         model_id=heating_snapshot.model_id, 
         sensor_id=heating_snapshot.sensor_id,
         zero_emission_ratio=heating_snapshot.zero_emission_ratio,
         insertion_date=collected_date, 
         insertion_time=collected_time)



from __future__ import print_function

import argparse
import ast
import os
import sys


# Parse arguments

parser = argparse.ArgumentParser()
parser.add_argument('--promise-folder', required=True)
parser.add_argument('--legacy-promise-folder', default=None)
parser.add_argument('--promise-timeout', type=int, default=20)
parser.add_argument('--partition-folder', default=None)
parser.add_argument('--log-folder', default=None)
parser.add_argument('--force', action='store_true')
parser.add_argument('--check-anomaly', action='store_true')
parser.add_argument('--debug', action='store_true')
parser.add_argument('--master-url', default=None)
parser.add_argument('--partition-cert', default=None)
parser.add_argument('--partition-key', default=None)
parser.add_argument('--partition-id', default=None)
parser.add_argument('--computer-id', default=None)
args = parser.parse_args()


# Extract slapos.core path and all dependencies from first promise found
# to import slapos.core

promise_folder = args.promise_folder
promise_file = next(
  p for p in os.listdir(promise_folder)
  if p.endswith('.py') and not p.startswith('__init__')
)
with open(os.path.join(promise_folder, promise_file)) as f:
  promise_content = f.read()
tree = ast.parse(promise_content, mode='exec')

sys.path[0:0] = eval(compile(ast.Expression(tree.body[1].value), '', 'eval'))

from slapos.grid.promise import PromiseLauncher, PromiseError
from slapos.cli.entry import SlapOSApp


# Configure promise launcher
# with the same logger as standard slapos command

app = SlapOSApp()
app.options, _ = app.parser.parse_known_args([])
app.configure_logging()

config = {k.replace('_', '-') : v for k, v in vars(args).items()}
promise_checker = PromiseLauncher(config=config, logger=app.log)


# Run promises
# Redirect stdout to stderr (logger only uses stderr already)
# to reserve stdout for error reporting

out = os.dup(1)
os.dup2(2, 1)

try:
  promise_checker.run()
except Exception as e:
  if sys.version_info < (3,):
    error_str = unicode(str(e), 'utf-8', 'repr')
  else:
    error_str = str(e)
  os.write(out, error_str.encode('utf-8', 'repr'))
  sys.exit(2 if isinstance(e, PromiseError) else 1)



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010-2014 Vifib SARL and Contributors.
# All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from __future__ import print_function

import subprocess
from six.moves.urllib.parse import urlparse
from time import sleep
import glob
import os
import netifaces
import socket
from netaddr import valid_ipv4, valid_ipv6
from slapos.cli.command import check_root_user
from slapos.cli.entry import SlapOSApp
from slapos.cli.config import ConfigCommand
from slapos.format import isGlobalScopeAddress
from slapos.util import string_to_boolean
import argparse
import logging

logger = logging.getLogger("slapos.boot")

def _removeTimestamp(instancehome, partition_base_name):
    """
    Remove .timestamp from all partitions
    """
    timestamp_glob_path = os.path.join(
        instancehome,
        "%s*" % partition_base_name,
        ".timestamp")
    for timestamp_path in glob.glob(timestamp_glob_path):
       logger.info("Removing %s", timestamp_path)
       os.remove(timestamp_path)


def _runBang(app):
    """
    Launch slapos node format.
    """
    logger.info("[BOOT] Invoking slapos node bang...")
    result = app.run(['node', 'bang', '-m', 'Reboot'])
    if result == 1:
      return 0
    return 1


def _runFormat(app):
    """
    Launch slapos node format.
    """
    logger.info("[BOOT] Invoking slapos node format...")
    result = app.run(['node', 'format', '--now', '--verbose'])
    if result == 1:
      return 0
    return 1


def _ping(hostname):
    """
    Ping a hostname
    """
    logger.info("[BOOT] Invoking ipv4 ping to %s...", hostname)
    p = subprocess.Popen(["ping", "-c", "2", hostname],
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    stdout, stderr = p.communicate()
    if p.returncode == 0:
      logger.info("[BOOT] IPv4 network reachable...")
      return 1
    logger.error("[BOOT] IPv4 network unreachable...")
    return 0


def _ping6(hostname):
    """
    Ping an ipv6 address
    """
    logger.info("[BOOT] Invoking ipv6 ping to %s...", hostname)
    p = subprocess.Popen(
        ["ping6", "-c", "2", hostname],
        stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, stderr = p.communicate()
    if p.returncode == 0:
        logger.info("[BOOT] IPv6 network reachable...")
        return 1
    logger.error("[BOOT] IPv6 network unreachable...")
    return 0


def _test_ping(hostname):
  is_ready = _ping(hostname)
  while is_ready == 0:
    sleep(5)
    is_ready = _ping(hostname)


def _test_ping6(hostname):
  is_ready = _ping6(hostname)
  while is_ready == 0:
    sleep(5)
    is_ready = _ping6(hostname)


def _ping_hostname(hostname):
  is_ready = _ping6(hostname)
  while is_ready == 0:
    sleep(5)
    # Try ping on ipv4
    is_ready = _ping(hostname)
    if is_ready == 0:
      # try ping on ipv6
      is_ready = _ping6(hostname)


def _waitIpv6Ready(ipv6_interface):
  """
    test if ipv6 is ready on ipv6_interface
  """
  logger.info("[BOOT] Checking if %r has IPv6...", ipv6_interface)
  while True:
    for inet_dict in netifaces.ifaddresses(ipv6_interface).get(socket.AF_INET6, ()):
      ipv6_address = inet_dict['addr'].split('%')[0]
      if isGlobalScopeAddress(ipv6_address):
        return

    logger.error("[BOOT] No IPv6 found on interface %r, "
          "try again in 5 seconds...", ipv6_interface)
    sleep(5)

class BootCommand(ConfigCommand):
    """
    Test network and invoke simple format and bang (Use on Linux startup)
    """
    command_group = 'node'

    def get_parser(self, prog_name):
        ap = super(BootCommand, self).get_parser(prog_name)
        ap.add_argument('-m', '--message',
                        default="Reboot",
                        help='Message for bang')
        return ap

    def take_action(self, args):
        configp = self.fetch_config(args)
        instance_root = configp.get('slapos','instance_root')
        partition_base_name = "slappart"
        if configp.has_option('slapformat', 'partition_base_name'):
          partition_base_name = configp.get('slapformat', 'partition_base_name')
        master_url = urlparse(configp.get('slapos','master_url'))
        master_hostname = master_url.hostname

        root_check = True
        if configp.has_option('slapos', 'root_check'):
            root_check = configp.getboolean('slapos', 'root_check')

        if root_check:
            check_root_user(self)

        # Check that we have IPv6 ready
        if configp.has_option('slapformat', 'ipv6_interface'):
            ipv6_interface = configp.get('slapformat', 'ipv6_interface')
        elif configp.has_option('slapformat', 'interface_name'):
            ipv6_interface = configp.get('slapformat', 'interface_name')
        else:
            # It is most likely the we are running on unpriviledged environment
            # so we for slapformat handle it.
            ipv6_interface = None

        if ipv6_interface is not None:
            _waitIpv6Ready(ipv6_interface)

        # Check that node can ping master
        if valid_ipv4(master_hostname):
            _test_ping(master_hostname)
        elif valid_ipv6(master_hostname):
            _test_ping6(master_hostname)
        else:
            # hostname
            _ping_hostname(master_hostname)

        app = SlapOSApp()
        # Make sure slapos node format returns ok
        while not _runFormat(app):
            logger.error("[BOOT] Fail to format, try again in 15 seconds...")
            sleep(15)

        # Make sure slapos node bang returns ok
        while not _runBang(app):
            logger.error("[BOOT] Fail to bang, try again in 15 seconds...")
            sleep(15)

        _removeTimestamp(instance_root, partition_base_name)
